"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var parse = function (file) { return new Promise(function (resolve, _reject) {
    var header = [];
    var scalars = [];
    var schema = {};
    var resetCurrent = function () { return ({
        description: [],
        fields: [],
        name: '',
        rest: [],
        type: '',
    }); };
    var lineReader = require('readline').createInterface({
        input: fs_1.default.createReadStream(file),
    });
    var current = resetCurrent();
    var seenNonComment = false;
    lineReader.on('line', function (line) {
        if (!line.trim()) {
            return;
        }
        if (line.startsWith('#')) {
            if (!seenNonComment) {
                return;
            }
            current.description.push(line);
            return;
        }
        seenNonComment = true;
        if (line.startsWith('scalar')) {
            scalars.push(line);
        }
        if (line.endsWith('{')) {
            var m = line.match(/(\w+)\b/g);
            if (m) {
                var type = m[0], name_1 = m[1], rest = m.slice(2);
                current.type = type;
                current.name = name_1;
                current.rest = rest;
                return;
            }
        }
        var endCurly = line.search('}');
        var comment = line.search('#');
        if (endCurly > -1 && (comment === -1 || endCurly < comment)) {
            var type = current.type, name_2 = current.name;
            if (!type) {
                return;
            }
            if (!schema[type]) {
                schema[type] = {};
            }
            schema[type][name_2] = current;
            current = resetCurrent();
            return;
        }
        if (!current.type) {
            current.description.push(line);
            return;
        }
        current.fields.push(line);
    });
    lineReader.on('close', function () { return resolve({ schema: schema, header: header, scalars: scalars }); });
}); };
var filterSchema = function (schemaPath, out, fieldFilters, _a) {
    var descriptions = _a.descriptions, _b = _a.removeCommaInDescription, removeCommaInDescription = _b === void 0 ? true : _b;
    return parse(schemaPath)
        .then(function (_a) {
        var schema = _a.schema;
        var bkFile = '';
        var formatDescription = function (desc) {
            var stripped = removeCommaInDescription ? desc.replace(/,/g, '') : desc;
            return "\"\"\" " + stripped + "\"\"\"";
        };
        var schemaTypes = Object.keys(schema);
        var output = '';
        for (var _i = 0, schemaTypes_1 = schemaTypes; _i < schemaTypes_1.length; _i++) {
            var schemaType = schemaTypes_1[_i];
            var typeNames = schema[schemaType];
            var _loop_1 = function (typeName) {
                var _a = typeNames[typeName], description = _a.description, fields = _a.fields, rest = _a.rest;
                var addition = [];
                var filteredFields = fields;
                var _loop_2 = function (filter) {
                    var transformField = filter.transformField, add = filter.add, exceptName = filter.exceptName, except = filter.except, invert = filter.invert, type = filter.type, line = filter.line, name_3 = filter.name, fieldName = filter.fieldName, custom = filter.custom;
                    if (type && type !== schemaType) {
                        return "continue";
                    }
                    if (name_3 && !fieldTest(name_3, typeName, exceptName)) {
                        return "continue";
                    }
                    if (add) {
                        addition.push(add);
                    }
                    if (transformField) {
                        filteredFields = filteredFields.map(function (fieldLine) {
                            var fieldSplitted = fieldLine.match(/(\w*):\s(\w*)/);
                            var _a = fieldSplitted || ['', '', ''], _ = _a[0], lineName = _a[1], lineDef = _a[2];
                            var match = fieldTest(fieldName, lineName, except) || fieldTest(line, fieldLine, except);
                            if (!match) {
                                return fieldLine;
                            }
                            var prepend = transformField.prepend, append = transformField.append, customTransform = transformField.custom;
                            if (customTransform) {
                                return customTransform({
                                    lineDef: lineDef,
                                    lineName: lineName,
                                    type: type,
                                    line: fieldLine,
                                    name: typeName,
                                });
                            }
                            return [prepend, fieldLine, append].filter(function (a) { return a; }).join('\n');
                        });
                    }
                    else {
                        filteredFields = filteredFields.filter(function (fieldLine) {
                            var fieldSplitted = fieldLine.match(/(\w*):\s(\w*)/);
                            var _a = fieldSplitted || ['', '', ''], _ = _a[0], lineName = _a[1], lineDef = _a[2];
                            var remove = custom
                                ? custom({
                                    lineDef: lineDef,
                                    lineName: lineName,
                                    type: type,
                                    line: fieldLine,
                                    name: typeName,
                                })
                                : (fieldTest(fieldName, lineName, except) || fieldTest(line, fieldLine, except));
                            return remove === !!invert;
                        });
                    }
                };
                for (var _i = 0, fieldFilters_1 = fieldFilters; _i < fieldFilters_1.length; _i++) {
                    var filter = fieldFilters_1[_i];
                    _loop_2(filter);
                }
                var previous = '';
                var openBrace = false;
                var nodeDescriptions = descriptions && descriptions[schemaType] && descriptions[schemaType][typeName];
                var reducedFields = filteredFields.reduce(function (r, line) {
                    var trimmed = line.trim();
                    if (trimmed.startsWith('#')) {
                        previous = '  ' + formatDescription(trimmed.replace('#', ''));
                        r.push(previous);
                        return r;
                    }
                    var match = trimmed.match(/^(\w*)([\:(])/);
                    var _a = match || ['', '', ''], _ = _a[0], firstWord = _a[1], colonBrace = _a[2];
                    if (colonBrace === '(') {
                        openBrace = true;
                    }
                    if (openBrace && line.search(/\):\s/) > -1) {
                        openBrace = false;
                    }
                    if (!openBrace && !!nodeDescriptions && !!nodeDescriptions[firstWord]) {
                        previous = "  " + formatDescription(nodeDescriptions[firstWord]) + '\n' + line;
                        r.push(previous);
                        return r;
                    }
                    previous = line;
                    r.push(previous);
                    return r;
                }, []);
                var nodeDescription = nodeDescriptions && nodeDescriptions['__node'];
                if (nodeDescription) {
                    description = [formatDescription(nodeDescription)];
                }
                output += "\n" + description
                    .filter(Boolean)
                    .join('\n') + "\n" + schemaType + " " + typeName + " " + rest + " {\n" + addition.concat(reducedFields).join('\n') + "\n}";
            };
            for (var _b = 0, _c = Object.keys(typeNames); _b < _c.length; _b++) {
                var typeName = _c[_b];
                _loop_1(typeName);
            }
        }
        output = output.replace(/input\s*\w*\s*{\s*}/, '');
        if (fs_1.default.existsSync(out)) {
            bkFile = path_1.default.join(out, '..', '.bk.' + path_1.default.basename(out));
            fs_1.default.copyFileSync(out, bkFile);
        }
        fs_1.default.writeFileSync(out, output);
        return { out: out, bkFile: bkFile };
    });
};
var fieldTest = function (filter, field, except) {
    if (!filter) {
        return false;
    }
    if (Array.isArray(filter)) {
        return filter.includes(field);
    }
    return field.search(filter) > -1 && (!field || !fieldTest(except, field));
};
exports.default = filterSchema;
//# sourceMappingURL=index.js.map